#!/usr/bin/env python
"""autodialer-au.py (v0.1) (2006-01-16)
by micolous <http://micolous.id.au>
idea based on autodialler.pl by NotTheory <http://www.oldskoolphreak.com>

Syntax:
  autodialler-au.py <start number> <end number> [log file]
  
 <start number>  : The number to start scanning from, eg: 1800123456.  Must
                   NOT contain spaces or dashes.
 <end number>    : The number to end at, eg: 1800234567.  Again, must NOT
                   contain spaces or dashes.
 [log file]      : It may be desirable to have a template file with all the
                   dialled numbers listed.  This will create a simple
                   template for you to fill in with the details of the
                   numbers that you're calling.

Description:
This autodials via the manager API on an Asterisk box.  This also will rip 
the details out of the ACMA allocated number database, so we only have to 
scan numbers that are allocated (dramatically reducing search time in the 
case of 1800/1300 numbers).  This is intended for use in the scanning of 
Australian phone numbers, however countries with a similar database to what
the ACMA offers could be adapted to use this script.  Of course, the really
really secret unlisted numbers won't be listed in the ACMA database, if you
are intending to go and look for those only.

This will also record the calls, so that if you end up with someone very
difficult to understand, that you can go back over it later and have
another listen without redialling.  Please note that you may not be able to
actually share those recordings due to certain wiretap laws.

You'll need a copy of the ACMA allocated number database, from [1].  You'll
need to convert this to SQLite3 format using converter.py.  This creates 
"acma.sqlite", which needs to be in the same directory as this script.
Because the database is updated daily, it is far easier to simply provide 
the conversion script.  Additionally, it saves my bandwidth! :)

This was ported to python, because python is awesome, and stuff. 8)  The
original (perl) script can be downloaded from [2].

References:
[1]: http://web.acma.gov.au/numb/openAccess/inquiry/viewAllocationSearch.do
[2]: http://www.oldskoolphreak.com/tfiles/voip/autodialer.txt
"""


import telnetlib, sys, apsw, time, ConfigParser

configFile = "autodialer-au.ini"

# load the config
conf = ConfigParser.ConfigParser()
conf.read(configFile)

host = conf.get("autodialler", "host")
port = int(conf.get("autodialler", "port"))
username = conf.get("autodialler", "username")
secret = conf.get("autodialler", "secret")
myPhone = conf.get("autodialler", "my-phone")
acmaDatabase = conf.get("autodialler", "acma-database")


now = time.strftime("%Y-%m-%d")

if len(sys.argv) < 3:
	print __doc__
	sys.exit(1)
	
report = ""
if len(sys.argv) >= 4:
	report = sys.argv[3]

start = sys.argv[1]
end = sys.argv[2]

print "Attempting to connect to " + host + ":" + str(port) + "..."
try:
	mgrlnk = telnetlib.Telnet(host, port)
except:
	print "There was an error trying to connect to the remote Asterisk server.  Maybe the server isn't running, or you have specified incorrect details?"
	sys.exit(1)

print "Waiting for Asterisk Call Manager header..."

mgrlnk.read_until("Asterisk Call Manager",10)
mgrlnk.read_until("\n")

print "Recieved Asterisk Management header.  Authenticating to server..."

mgrlnk.write("Action: Login\r\n")
mgrlnk.write("Username: " + username + "\r\n")
mgrlnk.write("Secret: " + secret + "\r\n")
mgrlnk.write("\r\n")

mgrlnk.read_until("Response: ")
buf = mgrlnk.read_until("\n")[:-2]
if buf == "Error":
	print "Error authenticating to Asterisk."
	print mgrlnk.read_until("\n")
	mgrlnk.close()
	sys.exit(1)

print "Authentication success!"
	
print "Attempting to open database file..."
con = apsw.Connection(acmaDatabase)
cur = con.cursor()

print "Database opened successfully.  Querying database..."
res = cur.execute("SELECT * FROM numbers WHERE (start >= :start) AND (end <= :end) AND (allocatee != '(SPARE)');", {"start": start, "end": end})

# sqlite doesn't give us a real 2D list like we want, so we have to do it ourself
result = []
for row in res:
	result.append(row)


if len(result)==0:
	print "Error, database returned no results for range", start, "->", end, "."
	print "Did you make the start number greater than the end number, or specify part of singular allocated range?"
	sys.exit(1)

print "Got " + str(len(result)) + " rows from database.  Iterating through result set..."

numbers = []

for row in result:
	if int(row[5]) == 1:
		numbers.append(row[3])
	else:
		for x in range(int(row[3]), int(row[4]) + 1):
			numbers.append(x)

print "Iteration complete.  Got " + str(len(numbers)) + " numbers to dial."

if len(report) > 1:
	print "Writing report template to", report, "..."
	fh = open(report, "w")
	fh.write("Scan for range " + start + " => " + end + """
Date: """ + now + """
Contributed by: You <youremail@example.com>
Range Size: """ + str(int(end) - int(start)) + " (" + str(len(numbers)) + """ allocated)

""")
	for num in numbers:
		fh.write(str(num) + ": \n")
	fh.close()
	print "Done writing report template."
	
print "Starting dialling sequence..."
for num in numbers:
	print "Attempting to call " + str(num) + "..."
	# Create call channel
	mgrlnk.write("Action: Originate\r\n")
	mgrlnk.write("Channel: " + myPhone + "\r\n")
	mgrlnk.write("Exten: " + str(num) + "\r\n")
	mgrlnk.write("Priority: 1\r\n")
	# some clients don't display the caller ID number, only the name (like sjphone)
	# so we add this in.
	mgrlnk.write("Callerid: \"Automatic call from autodialler-au.py " + str(num) + "\" <" + str(num) + ">\r\n")
	mgrlnk.write("\r\n")
	
	# Wait for success
	mgrlnk.read_until("Response: ")
	buf = mgrlnk.read_until("\n")[:-2]
	if buf == "Success":
		# search for channel name
		mgrlnk.read_until("Event: Newchannel")
		mgrlnk.read_until("Channel: ")
		chan = mgrlnk.read_until("\n")[:-2]
		
		print "New call created on " + chan + ".  Attempting to create monitoring on extension..."
		timestamp = time.strftime("%Y%m%d-%H%M%S")
		
		mgrlnk.write("Action: Monitor\r\n")
		mgrlnk.write("Channel: " + chan + "\r\n")
		mgrlnk.write("File: wardial-" + str(num) + "-" + timestamp + "\r\n")
		mgrlnk.write("Mix: 1\r\n")
		mgrlnk.write("\r\n")
		
		mgrlnk.read_until("Response: ")
		buf = mgrlnk.read_until("\n")[:-2]
		if buf == "Success":
			print "Monitor mode enabled successfully.  Writing to wardial-" + str(num) + "-" + timestamp + ".wav"
		else:
			print "Error enabling monitor mode."
			mgrlnk.read_until("Message: ")
			print "Message: " + mgrlnk.read_until("\n")[:-2]
			sys.exit(1)
		
		print "Waiting until termination..."
		mgrlnk.read_until("Event: Hangup")
		
		print "Call terminated, moving to next number..."
		
	else:
		print "Error creating call!"
		sys.exit(1)

print "Completed scans successfully!  You may wish to clean up the WAVE files on your Asterisk server now, because they take up lots of space (1MB per minute).  They're normally in /var/spool/asterisk/monitor/wardial-*"
