#!/usr/bin/env python
"""converter.py
This script converts the ACMA database to SQLite format
Copyright 2005 micolous

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

### IMPORT MODULES ###
import csv, os, sys
try:
	import apsw
except:
	print "Error, you must have the python-apsw package installed to use this program."
	sys.exit(1)

### OPEN InquiryFullDownload.csv ###
print "Opening InquiryFullDownload.csv..."
try:
	dbfh = open("InquiryFullDownload.csv", "rb")
	reader = csv.reader(dbfh)
except:
	print "Error, couldn't open 'InquiryFullDownload.csv' for reading!"
	print "Please download it from http://web.acma.gov.au/numb/openAccess/inquiry/viewAllocationSearch.do"
	print "and then decompress the file into the acmaphonesearch directory."
	sys.exit(1)
	
### MOVE OLD VERSION OUT OF THE WAY ###
print "Moving (possible) old database file out of the way..."
try:
	os.rename("acma.sqlite", "acma-old.sqlite")
except:
	# error could be permission denied, but we really do not know! oh well.
	print "Existing acma.sqlite not found."
else:
	print "Backed up acma.sqlite to acma-old.sqlite."


### CONNECT TO SQLite ###
print "Creating database..."
try:
	con = apsw.Connection("acma.sqlite")
	cur = con.cursor()
	cur.execute("""
		BEGIN;
		CREATE TABLE numbers (
			service_type,
			prefix,
			length,
			start,
			end,
			quantity,
			allocatee,
			allocation_date,
			holder,
			holder_date,
			numbering_area,
			exchange_area
		);
		
		CREATE TABLE holders (
			holder_name
		);
		
		CREATE TABLE services (
			service_type
		)

	""") # end sql
except:
	print "Error connecting to database file.  The error was:"
	print sys.exc_info()[0]
	sys.exit(1)

### IMPORT ROWS ###
service_types = []
holders = []
row_count = 0
print "Importing data (this will take some time)..."
for row in reader:
	# skip the first row, it's headers.
	if row_count > 0:
		# check the service type
		try:
			service_types.index(row[0])
		except:
			# then it doesn't exist yet.
			service_types.append(row[0])
		
		# check the holder and allocatee name
		try: # allocatee
			holders.index(row[6])
		except:
			holders.append(row[6])
			
		try: # holder
			holders.index(row[8])
		except:
			holders.append(row[8])
			
		# now go and add the entry to the DB
		cur.execute('INSERT INTO numbers VALUES (:svtype, :prefix, :length, :start, :end, :quantity, :alloc, :allocdate, :holder, :holderdate, :numarea, :excharea)', {'svtype': row[0], 'prefix': row[1], 'length': row[2], 'start': row[3], 'end': row[4], 'quantity': row[5], 'alloc': row[6], 'allocdate': row[7], 'holder': row[8], 'holderdate': row[9], 'numarea': row[10], 'excharea': row[11]})
		
	row_count = row_count + 1
	# every 5000 rows, post an update.
	if (row_count % 5000 == 0):
		print str(row_count) + " rows done..."
		
print str(row_count) + " rows done (complete!)"

### INDEX HOLDERS AND SERVICE TYPES ###
print "Indexing holders and services..."
for holder in holders:
	cur.execute('INSERT INTO holders VALUES (:holder)', {'holder': holder})

for service in service_types:
	cur.execute('INSERT INTO services VALUES (:service)', {'service': service})


### COMMIT ###
print "Committing to disk..."
cur.execute("COMMIT")

### CLEANUP ###
print "Cleanup..."
dbfh.close()
del reader
